#include <windows.h>
#include <stdio.h>

#include "Algorithm.h"
#include "constants.h"
#include "Main.h"

DWORD dwHashA;
DWORD dwHashB;

DWORD dwCryptTable[0x500];
void InitCryptTable()
{
  DWORD seed = 0x00100001;

  for( DWORD index1 = 0; index1 < 0x100; ++index1 )
  {
    for( DWORD index2 = index1, i = 0; i < 5; i++, index2 += 0x100 )
    {
      seed  = (seed * 125 + 3) % 0x2AAAAB;
      DWORD temp1 = (seed & 0xFFFF) << 0x10;

      seed  = (seed * 125 + 3) % 0x2AAAAB;
      DWORD temp2 = (seed & 0xFFFF);

      dwCryptTable[index2] = (temp1 | temp2);
    }
  }

  FILE *out = fopen("cryptTable.txt", "w+");
  if ( out )
  {
    for ( int i = 256; i < 512; ++i )
    {
      if ( !(i % 8) )
        fprintf(out, "\n");
      fprintf(out, "0x%08X, ", dwCryptTable[i] );
    }
    fprintf(out, "\n\n\n");
    for ( int i = 512; i < 768; ++i )
    {
      if ( !(i % 8) )
        fprintf(out, "\n");
      fprintf(out, "0x%08X, ", dwCryptTable[i] );
    }
    fclose(out);
    printf("Wrote cryptTable.\n");
  }
}

char szFoundStr[16];
DWORD dwFalseCount;
bool Found(DWORD *pdwString)
{
  ++dwFalseCount;
  DWORD dwSeed1 = prefixSeedB1;
  DWORD dwSeed2 = prefixSeedB2;
  DWORD *pdwStr = pdwString;
  while ( *pdwStr )
  {
    dwSeed1 = dwHashNameBTable[*pdwStr] ^ (dwSeed1 + dwSeed2);
    dwSeed2 = *pdwStr + dwSeed1 + dwSeed2 + (dwSeed2 << 5);
    ++pdwStr;
  }
  if ( dwSeed1 == dwHashB )
  {
    for ( int i = 0; pdwString[i]; ++i )
      szFoundStr[i] = (char)pdwString[i]-3;
    found = true;
    return true;
  }
  return false;
}

bool HashLoop(DWORD *pdwString, DWORD dwDepth, DWORD dwSeedA1, DWORD dwSeedA2)
{
  if ( dwDepth == MAXDEPTH )
    return false;

  for ( DWORD dwChar = 'A'+3; dwChar < 'Z'+4; ++dwChar )
  {
    DWORD dwNewSeedA1 = dwHashNameATable[dwChar] ^ (dwSeedA1 + dwSeedA2);
    DWORD dwNewSeedA2 = dwChar + dwNewSeedA1 + dwSeedA2 + (dwSeedA2 << 5);

    pdwString[dwDepth] = dwChar;
    if ( HashLoop(pdwString, dwDepth+1, dwNewSeedA1, dwNewSeedA2) )
      return true;

    DWORD *pdwSuffix = szSuffix;
    while ( *(pdwSuffix+1) )
    {
      DWORD dwSuffix = *pdwSuffix++;
      dwNewSeedA1 = dwHashNameATable[dwSuffix] ^ (dwNewSeedA1 + dwNewSeedA2);
      dwNewSeedA2 = dwSuffix + dwNewSeedA1 + dwNewSeedA2 + (dwNewSeedA2 << 5);
    }
    dwNewSeedA1 = dwHashNameATable[*pdwSuffix] ^ (dwNewSeedA1 + dwNewSeedA2);

    if ( dwNewSeedA1 == dwHashA )
    {
      memcpy(&pdwString[dwDepth + 1], szSuffix, 16*4);
      if ( Found(pdwString) )
        return true;
    }
  }
  return false;
}

bool HashLoopRestricted(DWORD *pdwString, DWORD dwDepth, DWORD dwSeedA1, DWORD dwSeedA2, DWORD dwOrderFlags)
{
  if ( dwDepth == MAXDEPTH )
    return false;

  DWORD dwFirstChr = (dwOrderFlags & 1) ? szBefore[dwDepth]  : 'A'+3;
  DWORD dwLastChr  = (dwOrderFlags & 2) ? szAfter[dwDepth]+1 : 'Z'+4;
  for ( DWORD dwChar = dwFirstChr; dwChar < dwLastChr; ++dwChar )
  {
    DWORD dwNewSeedA1 = dwHashNameATable[dwChar] ^ (dwSeedA1 + dwSeedA2);
    DWORD dwNewSeedA2 = dwChar + dwNewSeedA1 + dwSeedA2 + (dwSeedA2 << 5);

    pdwString[dwDepth] = dwChar;
    if ( (dwOrderFlags & 3)
         ? HashLoopRestricted(pdwString, dwDepth+1, dwNewSeedA1, dwNewSeedA2, ((dwChar == dwFirstChr && dwOrderFlags & 1) ? 1 : 0) | ((dwChar == dwLastChr && dwOrderFlags & 2) ? 2 : 0))
         : HashLoop(pdwString, dwDepth+1, dwNewSeedA1, dwNewSeedA2) )
      return true;

    DWORD *pdwSuffix = szSuffix;
    while ( *(pdwSuffix+1) )
    {
      DWORD dwSuffix = *pdwSuffix++;
      dwNewSeedA1 = dwHashNameATable[dwSuffix] ^ (dwNewSeedA1 + dwNewSeedA2);
      dwNewSeedA2 = dwSuffix + dwNewSeedA1 + dwNewSeedA2 + (dwNewSeedA2 << 5);
    }
    dwNewSeedA1 = dwHashNameATable[*pdwSuffix] ^ (dwNewSeedA1 + dwNewSeedA2);

    if ( dwNewSeedA1 == dwHashA )
    {
      memcpy(&pdwString[dwDepth + 1], szSuffix, 16*4);
      if ( Found(pdwString) )
        return true;
    }
  }
  return false;
}

DWORD HashStringToA(const char *lpszString)
{
  if ( !lpszString )
    return 0;

  DWORD seed1 = 0x7FED7FED;
  DWORD seed2 = 0xEEEEEEEE;
  while ( *lpszString )
  {
    int ch = toupper( *lpszString++ )+3;

    seed1 = dwHashNameATable[ch] ^ (seed1 + seed2);
    seed2 = ch + seed1 + seed2 + (seed2 << 5);
  }
  return seed1;
}

DWORD HashStringToB(const char *lpszString)
{
  if ( !lpszString )
    return 0;

  DWORD seed1 = 0x7FED7FED;
  DWORD seed2 = 0xEEEEEEEE;
  while ( *lpszString )
  {
    int ch = toupper( *lpszString++ )+3;

    seed1 = dwHashNameBTable[ch] ^ (seed1 + seed2);
    seed2 = ch + seed1 + seed2 + (seed2 << 5);
  }
  return seed1;
}
